//=========================================================================
//Copyright LizardOfOz.
//=========================================================================

::characterChangesClasses <- [];
::characterChanges <- {};
class CharacterChange
{
    lastFireTime = 0;
    lastWeapon = null;
    player = null;

    function constructor()
    {
    }

    function TryApply(player)
    {
        if (!IsValidPlayer(player))
            return;
        this.player = player;
        if (!CanApply() || !CheckTeam())
            return;
        if (!(player in characterChanges))
            characterChanges[player] <- [];
        characterChanges[player].push(this);
        OnApply();
    }

    function CanApply() { return true; }
    function CheckTeam() { return vscript.IsSurvivor(player); }
    function OnApply() { }
    function OnFrameTick() { }
    function OnTick(interval) { }
    function OnDamageDealt(params) { }
    function OnDamageTaken(params) { }
    function OnDeath(params) { }
    function OnHurtDealtEvent(victim, params) { }
}

function CharacterChanges_Spawn(player)
{
    characterChanges[player] <- [];
    foreach (characterChange in characterChangesClasses)
        try
        {
            local newChange = characterChange();
            newChange.TryApply.call(newChange, player);
        }
        catch(e) { }
}
AddListener("spawn", CharacterChanges_Spawn, -1);

function CharacterChanges_Tick(interval)
{
    foreach (player in GetAlivePlayers())
        if (player in characterChanges)
            foreach (characterChange in characterChanges[player])
                try { characterChange.OnTick.call(characterChange, interval); }
                catch(e) { }
}
AddListener("tick_always", CharacterChanges_Tick, 2);

function CharacterChanges_FrameTick(void)
{
    foreach (player in GetAlivePlayers())
        if (player in characterChanges)
            foreach (characterChange in characterChanges[player])
                try { characterChange.OnFrameTick.call(characterChange); }
                catch(e) { }
}
AddListener("tick_frame", CharacterChanges_FrameTick, 2);

function CharacterChanges_Death(params)
{
    local player;
    if ((player = GetPlayerFromParams(params)) == null)
        return;
    foreach (characterChange in characterChanges[player])
        try { characterChange.OnDeath.call(characterChange, params); }
        catch(e) { }
}
AddListener("death", CharacterChanges_Death, 0);

function CharacterChanges_Damage(params)
{
    local victim = params.const_entity;
    local attacker = params.attacker;

    if (IsValidPlayer(attacker))
        foreach (characterChange in characterChanges[attacker])
            try { characterChange.OnDamageDealt.call(characterChange, params); }
            catch(e) { }

    if (IsValidPlayer(victim))
        foreach (characterChange in characterChanges[victim])
            try { characterChange.OnDamageTaken.call(characterChange, params); }
            catch(e) { }
}
AddListener("damage_hook", CharacterChanges_Damage, 0);

function CharacterChanges_Hurt(params)
{
    local attacker = params[0];
    local victim = params[1];
    local eventParams = params[2];

    if (IsValidPlayer(attacker))
        foreach (characterChange in characterChanges[attacker])
            try { characterChange.OnHurtDealtEvent.call(characterChange, victim, eventParams); }
            catch(e) { }
}
AddListener("player_hurt", CharacterChanges_Hurt, 0);