//=========================================================================
//Copyright LizardOfOz.
//=========================================================================

function IsPlayerAlive(player)
{
    return GetPropInt(player, "m_lifeState") == LIFE_STATE.ALIVE;
}

function IsPlayerDead(player)
{
    return GetPropInt(player, "m_lifeState") != LIFE_STATE.ALIVE;
}

::IsValidPlayer <- function(player)
{
    try
    {
        return player != null && player.IsValid() && player.IsPlayer() && player.GetTeam() > 1;
    }
    catch(e)
    {
        return false;
    }
}

::IsNotValidPlayer <- function(player)
{
    try
    {
        return player == null || !player.IsValid() || !player.IsPlayer() || player.GetTeam() <= 1;
    }
    catch (e)
    {
        return true;
    }
}

function IsValidPlayerOrBuilding(entity)
{
    try
    {
        return entity != null
            && entity.IsValid()
            && entity.GetTeam() > 1
            && (entity.IsPlayer() || entity.GetClassname().find("obj_") == 0);
    }
    catch(e)
    {
        return false;
    }
}

function IsNotValidPlayerOrBuilding(entity)
{
    try
    {
        return entity == null
            || !entity.IsValid()
            || entity.GetTeam() <= 1
            || (!entity.IsPlayer() && entity.GetClassname().find("obj_") == null);
    }
    catch(e)
    {
        return true;
    }
}

function IsValidBuilding(building)
{
    try
    {
        return building != null
            && building.IsValid()
            && building.GetClassname().find("obj_") == 0
            && building.GetTeam() > 1;
    }
    catch(e)
    {
        return false;
    }
}

function GetAllValidPlayers()
{
    local result = [];
    for (local i = 1; i <= Constants.Server.MAX_PLAYERS; i++)
    {
        local player = PlayerInstanceFromIndex(i);
        if (IsValidPlayer(player))
            result.push(player);
    }
    return result;
}

function GetAllValidSurvivors()
{
    local result = [];
    for (local i = 1; i <= Constants.Server.MAX_PLAYERS; i++)
    {
        local player = PlayerInstanceFromIndex(i);
        if (IsSurvivor(player))
            result.push(player);
    }
    return result;
}

function GetAllValidZombies()
{
    local result = [];
    for (local i = 1; i <= Constants.Server.MAX_PLAYERS; i++)
    {
        local player = PlayerInstanceFromIndex(i);
        if (IsZombie(player))
            result.push(player);
    }
    return result;
}

function GetBuildings()
{
    local result = [];
    local building = null;
    while(building = Entities.FindByClassname(building, "obj_*"))
    {
        if (building.IsValid());
            result.push(building);
    }
    return result;
}

function IsSurvivorAlive(player)
{
    return IsSurvivor(player) && IsPlayerAlive(player);
}

function IsZombieAlive(player)
{
    return IsZombie(player) && IsPlayerAlive(player);
}

function IsSurvivor(player)
{
    return IsValidPlayer(player) && survivors.find(player) != null;
}

function IsZombie(player)
{
    return IsValidPlayer(player) && survivors.find(player) == null && player.GetTeam() > 1;
}

function GetSurvivorsTeam()
{
    return infection ? 3 : 2;
}

function GetZombiesTeam()
{
    return infection ? 2 : 3;
}

function GetAliveSurvivors()
{
    local result = [];
    for (local i = 1; i <= Constants.Server.MAX_PLAYERS; i++)
    {
        local player = PlayerInstanceFromIndex(i);
        if (IsSurvivorAlive(player))
            result.push(player);
    }
    return result;
}

function GetAliveZombies()
{
    local result = [];
    for (local i = 1; i <= Constants.Server.MAX_PLAYERS; i++)
    {
        local player = PlayerInstanceFromIndex(i);
        if (IsZombieAlive(player))
            result.push(player);
    }
    return result;
}

function GetAlivePlayers()
{
    local result = [];
    for (local i = 1; i <= Constants.Server.MAX_PLAYERS; i++)
    {
        local player = PlayerInstanceFromIndex(i);
        if (IsPlayerAlive(player))
            result.push(player);
    }
    return result;
}

function GetTotalPlayerCount()
{
    local aliveCount = 0;
    for (local i = 1; i <= Constants.Server.MAX_PLAYERS; i++)
        if (IsValidPlayer(PlayerInstanceFromIndex(i)))
            aliveCount++;
    return aliveCount;
}

function GetPlayerFromParams(params, key = "userid")
{
    if (!(key in params))
        return null;
    local player = GetPlayerFromUserID(params[key]);
    if (IsValidPlayer(player))
        return player;
    return null;
}

aliveSurvivors <- 0;

function TickSurvivorCounter(interval)
{
    aliveSurvivors = 0;
    for (local i = 1; i <= Constants.Server.MAX_PLAYERS; i++)
        if (IsSurvivor(PlayerInstanceFromIndex(i)))
            aliveSurvivors++;
}
AddListener("tick_always", TickSurvivorCounter, -100);

function GetSurvivorCount()
{
    return aliveSurvivors;
}

::PlaySoundForAll <- function(soundScript)
{
    for (local i = 1; i <= Constants.Server.MAX_PLAYERS; i++)
    {
        local player = PlayerInstanceFromIndex(i);
        if (IsValidPlayer(player))
            EmitSoundOnClient(soundScript, player);
    }
}

::CTFPlayer.Yeet <- function(vector)
{
    SetPropEntity(this, "m_hGroundEntity", null);
    this.ApplyAbsVelocityImpulse(vector);
    this.RemoveFlag(Constants.FPlayer.FL_ONGROUND);
}

::CTFBot.SwitchTeam <- function(team)
{
    this.ForceChangeTeam(team, true);
    SetPropInt(this, "m_iTeamNum", team);
}

::CTFPlayer.SwitchTeam <- function(team)
{
    SetPropInt(this, "m_bIsCoaching", 1);
    this.ForceChangeTeam(team, true);
    SetPropInt(this, "m_bIsCoaching", 0);
}

::SwitchPlayerTeam <- function(player, team)
{
    if (IsValidPlayer(player))
        player.SwitchTeam(team);
}